# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""This module contains interface functions for the LTFAT computed
versions of dst calculations.

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import cython
import numpy as np

from ltfat cimport ltfatInt, dst_kind, dst_cd, dst_d
from ltfatpy.comp.ltfat cimport DSTI, DSTII, DSTIII, DSTIV


# don’t check for out-of-bounds indexing.
@cython.boundscheck(False)
# assume no negative indexing.
@cython.wraparound(False)
cdef comp_dst_cd(const double complex[:] f, const ltfatInt L,
                 const ltfatInt W, const dst_kind kind, double complex[:] out):
    """ Internal function, do not use it """
    dst_cd(&f[0], L, W, &out[0], kind)

# don’t check for out-of-bounds indexing.
@cython.boundscheck(False)
# assume no negative indexing.
@cython.wraparound(False)
cdef comp_dst_d(const double[:] f, const ltfatInt L,
                const ltfatInt W, const dst_kind kind, double[:] out):
    """ Internal function, do not use it """
    dst_d(&f[0], L, W, &out[0], kind)

# don’t check for out-of-bounds indexing.
@cython.boundscheck(False)
# assume no negative indexing.
@cython.wraparound(False)
cpdef comp_dst(f, type):
    """Function that computes dst

    This is a computational subroutine, do not call it directly, use
    :func:`~ltfatpy.fourier.dst.dsti`, :func:`~ltfatpy.fourier.dst.dstii`,
    :func:`~ltfatpy.fourier.dst.dstiii` or func:`~ltfatpy.fourier.dst.dstiv`
    instead.
    """
    cdef ltfatInt L, W
    if (f.dtype.type != np.float64) and (f.dtype.type != np.complex128):
        raise TypeError("f data should be numpy.float64 or complex128")
    if type not in {1, 2, 3, 4}:
        raise TypeError("type should be 1, 2, 3 or 4")

    dst_type = {1: DSTI, 2: DSTII, 3: DSTIII, 4: DSTIV}

    if f.ndim > 1:
        if f.ndim > 2:
            f = np.squeeze(f, axis=range(2, f.ndim-1))
        L = f.shape[0]
        W = f.shape[1]
        f_combined = f.reshape(L * W, order='F')
    else:
        L = f.shape[0]
        W = 1
        f_combined = f

    if f.dtype.type == np.float64:
        res = np.ndarray((L * W), dtype=np.float64)
        comp_dst_d(f_combined, L, W, dst_type[type], res)
    else:
        res = np.ndarray((L * W), dtype=np.complex128)
        comp_dst_cd(f_combined, L, W, dst_type[type], res)
    if W > 1:
        res = np.reshape(res, (L, W), order='F')
    return res



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""This module contains DSTIV function

Ported from ltfat_2.1.0/fourier/dstiv.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

from ltfatpy.comp.comp_dst import comp_dst
from ltfatpy.comp.assert_sigreshape_pre import assert_sigreshape_pre
from ltfatpy.comp.assert_sigreshape_post import assert_sigreshape_post
from ltfatpy.tools.postpad import postpad


def dstiv(f, L=None, dim=None):
    """Discrete Sine Transform type IV

    - Usage:

        | ``c = dstiv(f)``
        | ``c = dstiv(f,L,dim)``

    - Input parameters:

    :param numpy.ndarray f: Input data. **f** dtype has to be float64 or
        complex128.
    :param int L: Length of the output vector. Default is the length of
        **f**.
    :param int dim: dimension along which the transformation is applied.
        Default is the first non-singleton dimension.

    - Output parameter:

        :return: ``c``
        :rtype: numpy.ndarray

    ``dstiv(f)`` computes the discrete sine transform of type I of the
    input signal **f**. If **f** is multi-dimensional, the transformation is
    applied along the first non-singleton dimension.

    ``dstiv(f,L)`` zero-pads or truncates **f** to length **L** before doing
    the transformation.

    ``dstiv(f,[],dim)`` or ``dstiv(f,L,dim)`` applies the transformation along
    dimension **dim**.

    The transform is real (output is real if input is real) and orthonormal.

    This transform is its own inverse.

    Let f be a signal of length **L** and let ``c=dstiv(f)``. Then

    .. math::

        c\\left(n+1\\right)=\\sqrt{\\frac{2}{L}}\\sum_{m=0}^{L-1}f\\left(
        m+1\\right)\\sin\\left(\\frac{\\pi}{L}\\left(n+\\frac{1}{2}\\right)
        \\left(m+\\frac{1}{2}\\right)\\right)

    - Examples:

    The following figures show the first 4 basis functions of the DSTIV of
    length 20:

    >>> import numpy as np
    >>> # The dstiv is its own adjoint.
    >>> F = dstiv(np.eye(20, dtype=np.float64))
    >>> import matplotlib.pyplot as plt
    >>> plt.close('all')
    >>> fig = plt.figure()
    >>> for ii in range(1,5):
    ...    ax = fig.add_subplot(4,1,ii)
    ...    ax.stem(F[:,ii-1])
    ...
    <Container object of 3 artists>
    <Container object of 3 artists>
    <Container object of 3 artists>
    <Container object of 3 artists>
    >>> plt.show()

    .. image:: images/dstiv.png
       :width: 700px
       :alt: dstiv image
       :align: center
    .. seealso::  :func:`~ltfatpy.fourier.dsti`,
        :func:`~ltfatpy.fourier.dstii`, :func:`~ltfatpy.fourier.dstiii`,
        :func:`~ltfatpy.fourier.dcti`

    - References:
        :cite:`rayi90,wi94`
    """
    (f, L, _, _, dim, permutedsize, order) = assert_sigreshape_pre(f, L, dim)
    if L is not None:
        f = postpad(f, L)
    if L == 1:
        c = f
    else:
        c = comp_dst(f, 4)
    return assert_sigreshape_post(c, dim, permutedsize, order)

if __name__ == '__main__':  # pragma: no cover
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Read .mat data files generated with the Matlab version of Ltfat and
containing windows and their corresponding tight gabor window for validation.

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import scipy.io


class GabTightExamples:
    """This class opens the file and provides :func:`read_next_frame`
    method for reading each (g, gt) tuples contained in the file.
    """

    def __init__(self, filename):
        """Constructor

        - Input parameter:

        :param str filename: Name of the matlab data file
        """
        self.Data = scipy.io.loadmat(filename, struct_as_record=False,
                                     squeeze_me=True)['Data']
        self.ind = 0
        self.size = self.Data.shape[0]

    def read_next_frame(self):
        """Read the next frame of the file.

        - Usage:

            | `(L, a, M, gl, G, GT) = GabTightExamples("file.mat").
            | read_next_signal()`

        - Output parameters:

        :returns:
            :literal:`(L, a, M, gl, G, GT)`

            Empty strings if all frames have been read.
        """
        if(self.ind >= self.size):
            return ('', '', '', '', '', '')

        L = self.Data[self.ind].L
        a = self.Data[self.ind].a
        M = self.Data[self.ind].M
        rname = self.Data[self.ind].rname
        G = self.Data[self.ind].g
        GT = self.Data[self.ind].gt
        self.ind += 1
        return (L, a, M, rname, G, GT)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


""" Module of derivative of smooth periodic function computation

Ported from ltfat_2.1.0/fourier/pderiv.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np

from ltfatpy.fourier.fftindex import fftindex
from ltfatpy.comp.assert_sigreshape_pre import assert_sigreshape_pre
from ltfatpy.comp.assert_sigreshape_post import assert_sigreshape_post


def pderiv(f, dim=None, difforder=4):
    """ Derivative of smooth periodic function

    - Usage:
        | ``fd = pderiv(f)``
        | ``fd = pderiv(f, dim)``
        | ``fd = pderiv(f, dim, difforder)``

    - Input parameters:

    :param numpy.ndarray f: Input array
    :param int dim: Axis over which to compute the derivative
    :param difforder: Order of the centered finite difference scheme used.
        Possible values are: ``2``, ``4``, ``float('inf')``
    :type difforder: int or float

    - Output parameters:

    :returns: Derivative of **f**
    :rtype: numpy.ndarray

    ``pderiv(f)`` will compute the derivative of **f** using a using a 4th
    order centered finite difference scheme. **f** must have been obtained by
    a regular sampling. If **f** is a matrix, the derivative along the
    columns will be found.

    ``pderiv(f, dim)`` will do the same along dimension **dim**.

    ``pderiv(f, dim, difforder)`` uses a centered finite difference scheme of
    order difforder instead of the default.

    ``pderiv(f, dim, float('inf'))`` will compute the spectral derivative
    using a DFT.

    ``pderiv`` assumes that **f** is a regular sampling of a function on the
    torus ``[0, 1)``. The derivative of a function on a general torus
    ``[0, T)`` can be found by scaling the output by ``1/T``.
    """

    f, L, Ls, W, dim, permutedsize, order = assert_sigreshape_pre(f, dim=dim)

    if difforder == 2:
        fd = L * (np.roll(f, -1, 0) - np.roll(f, 1, 0)) / 2
    elif difforder == 4:
        fd = L * (- np.roll(f, -2, 0) + 8*np.roll(f, -1, 0) -
                  8*np.roll(f, 1, 0) + np.roll(f, 2, 0)) / 12
    elif difforder == float('inf'):
        n = fftindex(L, 0)
        n = np.tile(n, (W, 1)).transpose()

        fd = 2*np.pi*np.fft.ifft(1j*n*np.fft.fft(f, axis=0), axis=0)

        if np.isrealobj(f):
            fd = np.real(fd)

    else:
        raise ValueError('The specified differentation order is not '
                         'implemented.')

    fd = assert_sigreshape_post(fd, dim, permutedsize, order)

    return fd



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""This module pads or truncates arrays

Ported from ltfat_2.1.0/mex/postpad.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np

from ltfatpy.comp.assert_sigreshape_pre import assert_sigreshape_pre
from ltfatpy.comp.assert_sigreshape_post import assert_sigreshape_post


def postpad(x, L, C=0., dim=None):
    """Pads or truncates an array to a specified length

    - Usage:

        | ``y = postpad(x, L)``
        | ``y = postpad(x, L, C)``
        | ``y = postpad(x, L, C, dim)``

    - Input parameters:

    :param numpy.ndarray x: Input array
    :param int L: Length of the output array
    :param float C: Value of the padded entries
    :param int dim: Axis over which to pad or truncate **x**

    - Ouput parameters:

    :returns: Padded or truncated array
    :rtype: numpy.ndarray

    ``postpad(x, L)`` will add zeros to the end of the vector **x**, until
    the result has length **L**. If **L** is less than the length of the
    signal, it will be truncated. ``postpad`` works along the first
    non-singleton dimension.

    ``postpad(x, L, C)`` will add entries with a value of **C** instead of
    zeros.

    ``postpad(x, L, C, dim)`` works along dimension **dim** instead of the
    first non-singleton.

    .. seealso::
        :func:`~ltfatpy.fourier.middlepad.middlepad`
    """

    if dim is None:
        # by default, dim is the first non-singleton dimension
        dim = int(np.nonzero(np.array(x.shape) > 1)[0][0])

    x, L, Ls, W, dim, permutedsize, order = assert_sigreshape_pre(x, L, dim)

    if Ls < L:
        tmp = np.empty((L-Ls, W), dtype=x.dtype)
        tmp[:] = C
        y = np.concatenate((x, tmp), axis=0)
    else:
        y = x[:L, :].copy()

    y = assert_sigreshape_post(y, dim, permutedsize, order)

    return y



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module of phaseunlocking

Ported from ltfat_2.1.0/gabor/phaseunlock.m

.. moduleauthor:: Florent Jaillet
"""

import numpy as np


def phaseunlock(c, a):
    """Undo phase lock of Gabor coefficients

    - Usage:

        | ``c_out = phaseunlock(c, a)``

    - Input parameters:

    :param numpy.ndarray c: phaselocked Gabor coefficients
    :param int a: Length of time shift

    - Output parameters:

    :returns: non-phaselocked Gabor coefficients
    :rtype: numpy.ndarray

    ``phaseunlock(c, a)`` removes phase locking from the Gabor coefficients
    **c**.
    The coefficient must have been obtained from a
    :func:`~ltfatpy.gabor.dgt.dgt` with parameter **a**.

    Phase locking the coefficients modifies them so as if they were obtained
    from a time-invariant Gabor system. A filter bank produces phase locked
    coefficients.

    .. seealso:: :func:`~ltfatpy.gabor.dgt.dgt`,
                 :func:`~ltfatpy.gabor.phaselock.phaselock`, :func:`symphase`

    - References:
        :cite:`puc95`
    """

    # NOTE: This function doesn't support the parameter lt (lattice type)
    # supported by the corresponding octave function and the lattice used is
    # seperable (square lattice lt = (0, 1)).

    if not isinstance(a, int):
        raise(TypeError('a must be an integer'))

    M = c.shape[0]
    N = c.shape[1]
    L = N * a
    b = L / M

    if b % 1 != 0.:
        raise(ValueError('Lattice error. The a parameter is probably '
                         'incorrect.'))

    TimeInd = np.arange(N) * a
    FreqInd = np.arange(M)

    phase = FreqInd[:, np.newaxis].dot(TimeInd[np.newaxis, :])
    phase = np.mod(phase, M)
    phase = np.exp(-2.*1.j*np.pi*phase/M)

    # Handle multisignals
    shape = np.array(c.shape)
    if shape.shape[0] > 2:
        shape[2:] = 1
    c_out = c * phase.reshape(shape)

    return c_out



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the comp_isepdgt function

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import unittest
import numpy as np

from ltfatpy.comp.comp_isepdgt import comp_isepdgt
from ltfatpy.comp.comp_isepdgtreal import comp_isepdgtreal
from ltfatpy.tests.datasets.read_dgt_signal_ex_mat import DgtSignals
from ltfatpy.tests.datasets.get_dataset_path import get_dataset_path


class TestCompISepdgt(unittest.TestCase):

    # Called before the tests.
    def setUp(self):
        self.filename = get_dataset_path('sepdgt_signal_ex.mat')

    # Called after the tests.
    def tearDown(self):
        print('Test done')

    def test_default(self):
        """ Comparing results with Matlab generated ones
        """
        dgs = DgtSignals(self.filename)
        (TYPE, PHASETYPE, L, W, a, M, gl, SIGNAL, WINDOW, DGT, DUAL_WINDOW,
         IDGT) = dgs.read_next_signal()
        while TYPE != '':
            mess = "\n\nTYPE = " + TYPE + "\nPHASETYPE = " + PHASETYPE
            mess += "\nL = " + str(L) + "\nW = " + str(W) + "\na = " + str(a)
            mess += "\nM = " + str(M) + "\ngl = " + str(gl) + "\nSIGNAL =\n"
            mess += str(SIGNAL) + "\nWINDOW =\n" + str(WINDOW)
            mess += "\nDGT\n" + str(DGT) + "\nDUAL_WINDOW\n"
            mess += str(DUAL_WINDOW) + "\nIDGT\n" + str(IDGT)
            pt = 0
            if (PHASETYPE == "timeinv"):
                pt = 1
            if (TYPE == "REAL"):
                frec = comp_isepdgtreal(DGT, DUAL_WINDOW, a, M, pt)
            else:
                frec = comp_isepdgt(DGT, DUAL_WINDOW, a, pt)
            mess += "frec = \n" + str(frec)
            self.assertTrue(np.linalg.norm(frec-IDGT) <= 1e-10, mess)
            (TYPE, PHASETYPE, L, W, a, M, gl, SIGNAL, WINDOW, DGT, DUAL_WINDOW,
             IDGT) = dgs.read_next_signal()

if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestCompISepdgt)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module of S0 norm calculation

Ported from ltfat_2.1.0/gabor/s0norm.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import numpy as np
from numpy import linalg as LA

from ltfatpy.comp.assert_sigreshape_pre import assert_sigreshape_pre
from ltfatpy.comp.assert_sigreshape_post import assert_sigreshape_post


def s0norm(f, rel=False, dim=None):
    """S0-norm of signal

    - Usage:

        | ``y = s0norm(f)``

    - Input parameters:

    :param numpy.ndarray f: is the signal
    :param bool rel: True if returning result should be relative to the
                        :math:`l^2` norm (the energy) of the signal.
                        False by default
    :param int dim: dimension along which norm is applied (first
                    non-singleton dimension as default)

    - Output parameters:

    :return: s0-norm
    :rtype: float

    ``s0norm(f)`` computes the :math:`S_0`-norm of a vector.

    If the input is a matrix or ND-array, the :math:`S_0`-norm is computed
    along the first (non-singleton) dimension, and a vector of values is
    returned.

    .. warning:: The :math:`S_0`-norm is computed by computing a full
        short-time Fourier transform of a signal, which can be quite
        time-consuming. Use this function with care for long signals.
    """
    #  ------ Computation --------------------------
    (f, L, _unused, W, dim, permutedsize, order) = assert_sigreshape_pre(
                                                    f, dim=dim)
    permutedsize_list = [1] + list(permutedsize[1:])
    permutedsize = tuple(permutedsize_list)
    if len(permutedsize_list) == 1:
        permutedsize_list.append(1)
    y = np.zeros(tuple(permutedsize_list), dtype=f.dtype)
    g = __call_pgauss(L)

    for ii in range(W):
        # Compute the STFT by the simple algorithm and sum each column of the
        # STFT as they are computed, to avoid L^2 memory usage.
        for jj in range(L):
            y[0, ii] = y[0, ii] + np.sum(np.abs(np.fft.fft(f[:, ii] *
                                         np.roll(g, jj, axis=0))))

        if rel:
            y[0, ii] = y[0, ii] / LA.norm(f[:, ii])

    y /= L
    y = assert_sigreshape_post(y, dim, permutedsize, order)
    return y


def __call_pgauss(L):
    from ltfatpy.fourier.pgauss import pgauss
    return pgauss(L)[0]



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module of glockenspiel signal loading

.. moduleauthor:: Denis Arrivault
"""


from scipy.io.wavfile import read as wavread
import importlib_resources  # not importlib.resources


import numpy as np


def gspi():
    """Load the 'glockenspiel' test signal

    - Usage:

        | ``(s, fs) = gspi()``

    - Output parameters:

    :returns: ``(s, fs)``
    :rtype: tuple

    :var numpy.ndarray s: 'glockenspiel' signal
    :var int fs: sampling frequency in Hz

    ``gspi`` loads the 'glockenspiel' signal. This is a recording of a simple
    tune played on a glockenspiel. It is 262144 samples long, mono, recorded
    at 44100 Hz using 16 bit quantization.

    This signal, and other similar audio tests signals, can be found on
    the EBU SQAM test signal CD http://tech.ebu.ch/publications/sqamcd.
    """

    # f = pkg_resources.resource_stream(__name__, "gspi.wav")
    # ref = ref = importlib.resources.files(__name__).joinpath("gspi.wav")
    ref = importlib_resources.files(__package__).joinpath("gspi.wav")

    with ref.open('rb') as f:
        fs, s = wavread(f)
    s = s.astype(np.float64) / 2.**15.
    return (s, fs)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


""" Module of even function tests

Ported from ltfat_2.1.0/fourier/isevenfunction.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import numpy as np
import numpy.linalg as LA


def isevenfunction(f, tol=1e-10, centering='wp'):
    """ True if function is even

    - Usage:

        | ``t = isevenfunction(f)``
        | ``t = isevenfunction(f, tol)``

    - Input parameters:

    :param numpy.ndarray f: vector of data to test (one dimension)
    :param float tol: tolerance (1e-10 by default)
    :param str centering: Point even function type : whole or half point even.
        **centering** can be 'wp' or 'hp', 'wp' is the default.

    -Output parameter:

    :return: True if **f** is whole point even
    :rtype: bool

    `isevenfunction(f)` returns True if *f* is whole point even. Otherwise it
    returns False.

    ``isevenfunction(f, tol)`` the same, using the tolerance *tol* to measure
    how large the error between the two parts of the vector can be. Default
    is 1e-10.

    Setting **centering** to 'hp', does the same for half point even functions.

    .. seealso:: :func:`~ltfatpy.fourier.middlepad.middlepad`, :func:`peven`
    """
    if f.ndim > 1:
        raise ValueError("f should be a one dimensional vector")

    # Define initial values for flags
    # definput.flags.centering = {'wp','hp'};
    # definput.keyvals.tol     = 1e-10;

    L = f.shape[0]

    if centering == 'wp':
        # Determine middle point of sequence.
        if L % 2 == 0:
            middle = L // 2
        else:
            middle = (L+1) // 2

        # Relative norm of difference between the parts of the signal.
        d = (LA.norm(f[1:middle] - np.conj(np.flipud(f[L-middle+1:L]))) /
             LA.norm(f))
    elif centering == 'hp':
        middle = int(np.floor(L/2))
        d = (LA.norm(f[0:middle] - np.conj(np.flipud(f[L-middle:L]))) /
             LA.norm(f))
    else:
        raise ValueError("centering parameter should be set to 'wp'" +
                         "(default) or 'hp'")

    # Return true if d less than tolerance.
    return (d <= tol)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module of linus signal loading

Ported from ltfat_2.1.0/signals/linus.m

.. moduleauthor:: Florent Jaillet
"""


from scipy.io.wavfile import read as wavread
import importlib_resources  # not importlib.resources

import numpy as np


def linus():
    """Load the 'linus' test signal

    - Usage:

        | ``(s, fs) = linus()``

    - Output parameters:

    :returns: ``(s, fs)``
    :rtype: tuple

    :var numpy.ndarray s: 'linus' signal
    :var int fs: sampling frequency in Hz

    ``linus`` loads the 'linus' signal. It is a recording of Linus Thorvalds
    pronouncing the words "Hello. My name is Linus Thorvalds, and I pronounce
    Linux as Linux".

    The signal is 41461 samples long and is sampled at 8 kHz.

    See `<http://www.paul.sladen.org/pronunciation/>`_.
    """

    # f = pkg_resources.resource_stream(__name__, "linus.wav")
    # ref = importlib.resources.files(__name__).joinpath("linus.wav")
    ref = importlib_resources.files(__package__).joinpath("linus.wav")
    with ref.open('rb') as f:
        fs, s = wavread(f)
    s = s.astype(np.float64) / 2.**15.
    return (s, fs)
    fs, s = wavread(f)
    s = s.astype(np.float64) / 2.**15.
    return (s, fs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########

import sys
import os
import shutil
from pathlib import Path


import distutils.spawn as ds
import distutils.dir_util as dd

# Always prefer setuptools over distutils
from setuptools import setup, Extension
from distutils.command.clean import clean
from distutils.command.sdist import sdist

# Test if Cython is installed
USE_CYTHON = True
try:
    from Cython.Distutils import build_ext
except ImportError:
    USE_CYTHON = False
    from distutils.command.build_ext import build_ext

USE_COPYRIGHT = True
try:
    from copyright import writeStamp, eraseStamp
except ImportError:
    USE_COPYRIGHT = False


###################
# Get ltfat version
####################
def get_version():
    v_text = open('VERSION').read().strip()
    v_text_formted = '{"' + v_text.replace('\n', '","').replace(':', '":"')
    v_text_formted += '"}'
    v_dict = eval(v_text_formted)
    return v_dict["ltfatpy"]


########################
# Set ltfat __version__
########################
def set_version(ltfat_dir, VERSION):
    filename = os.path.join(ltfat_dir, '__init__.py')
    buf = ""
    for line in open(filename, "rb"):
        if not line.decode("utf8").startswith("__version__ ="):
            buf += line.decode("utf8")
    f = open(filename, "wb")
    f.write(buf.encode("utf8"))
    f.write(('__version__ = "%s"\n' % VERSION).encode("utf8"))

def remove_generated_c_file():
    for dirpath, dirnames, filenames in os.walk('.'):
        for filename in filenames:
            if filename.endswith('.c'):
                full_path = os.path.join(dirpath, filename)
                # Optionnel : ne supprime que ceux qui ont un .pyx à côté
                pyx_path = full_path[:-2] + '.pyx'
                if os.path.exists(pyx_path):
                    os.remove(full_path)
#################
# CMake function
#################
def run_cmake(root_dir):
    """ Runs CMake to determine configuration for this build """
    if ds.find_executable('cmake') is None:
        print("CMake is required to build ltfatpy")
        print("Please install cmake version >= 2.6 and re-run setup")
        sys.exit(-1)
    print("Configuring ltfatpy build with CMake.... ")
    print("Root dir : " + root_dir)
    new_dir = os.path.join(root_dir, 'build')
    dd.mkpath(new_dir)
    os.chdir(new_dir)
    try:
        ds.spawn(['cmake', '..'])
    except ds.DistutilsExecError:
        print("Error while running cmake")
        print("run 'setup.py build --help' for build options")
        print("You may also try editing the settings in CMakeLists.txt file " + 
              "and re-running setup")
        sys.exit(-1)

###################
# CLEAN CMAKE CACHE
###################
def clean_stale_cmake_cache():
    cmake_cache = Path("build/CMakeCache.txt")
    if cmake_cache.exists():
        with cmake_cache.open("r", encoding="utf-8", errors="ignore") as f:
            content = f.read()
            if "CMAKE_HOME_DIRECTORY" in content:
                # Extract current path and stored source path
                import re
                match = re.search(r"CMAKE_HOME_DIRECTORY:INTERNAL=(.*)", content)
                if match:
                    recorded_source = os.path.abspath(match.group(1))
                    actual_source = os.path.abspath(".")
                    if recorded_source != actual_source:
                        print("Detected stale CMake cache (was: {})".format(recorded_source))
                        print("Removing 'build/' to avoid CMake path conflict.")
                        shutil.rmtree("build", ignore_errors=True)


#################
# make function
#################
def run_make(root_dir):
    """ Runs make to build ltfatpy libraries """
    print("Building ltfatpy libraries.... ")
    build_dir = os.path.join(root_dir, 'build')
    os.chdir(build_dir)
    try:
        ds.spawn(['make'])
    except ds.DistutilsExecError:
        print("Error while running make")
        print("run 'setup.py build --help' for build options")
        print("You may also try editing the settings in CMakeLists.txt file " + 
              "and re-running setup")
        sys.exit(-1)


#######################
# make install function
#######################
def run_make_install(root_dir):
    """ Runs make install to install ltfatpy libraries """
    print("Installing ltfatpy libraries.... ")
    build_dir = os.path.join(root_dir, 'build')
    os.chdir(build_dir)
    try:
        ds.spawn(['make', 'install'])
        # +cmake_args.split())
    except ds.DistutilsExecError:
        print("Error while running make install")
        print("run 'setup.py build --help' for build options")
        print("You may also try editing the settings in CMakeLists.txt file " + 
              "and re-running setup")
        sys.exit(-1)


#################
# uninstall libs
#################
def run_uninstall(root_dir):
    """ Un-installs ltfatpy libraries """
    print("Uninstall ltfatpy libraries.... ")
    build_dir = os.path.join(root_dir, 'build')
    os.chdir(build_dir)
    try:
        ds.spawn(['make', 'uninstall'])
    except ds.DistutilsExecError:
        print("Error while running make uninstall")
        print("run 'setup.py build --help' for build options")
        print("You may also try editing the settings in CMakeLists.txt file " + 
              "and re-running setup")
        sys.exit(-1)


#########################
# Custom 'build_ext' command
#########################
# class m_build_ext(build_ext):
#    """  Custom build_ext command """
#    def run(self):
#        root_dir = os.path.dirname(os.path.abspath(__file__))
#        cur_dir = os.getcwd()
#        run_cmake(root_dir)
#        run_make(root_dir)
#        run_make_install(root_dir)
#        os.chdir(cur_dir)
#        build_ext.run(self)

class m_build_ext(build_ext):
    def run(self):
        if USE_CYTHON:
            remove_generated_c_file()
        root_dir = os.path.dirname(os.path.abspath(__file__))
        cur_dir = os.getcwd()
        run_cmake(root_dir)
        run_make(root_dir)
        run_make_install(root_dir)
        os.chdir(cur_dir)
        build_ext.run(self)
##########################
# File path read command
##########################
def read(*paths):
    """Build a file path from *paths* and return the contents."""
    from io import open
    with open(os.path.join(*paths), 'r', encoding='utf-8') as f:
        return f.read()


#####################################
# Directory pyx files scan command
#####################################
def findpyxfiles(directory, files=[]):
    """scan a directory for pyx extension files."""
    for filename in os.listdir(directory):
        path = os.path.join(directory, filename)
        if os.path.isfile(path) and path.endswith(".pyx"):
            files.append(path.replace(os.path.sep, ".")[:-4])
        elif os.path.isdir(path):
            findpyxfiles(path, files)
    return files


#################
# Extension maker
#################
def makeExtension(extName, fileExt, lib_dir, path_includes):
    """Generate an Extension object from its dotted name."""
    extPath = extName.replace(".", os.path.sep) + fileExt
    print("Found " + extName + " extension...")
    return Extension(
        extName,
        [extPath],
        language="c",
        include_dirs=path_includes,
        extra_compile_args=['-O3'],
        libraries=["fftw3", "m", "blas", "lapack"],
        extra_objects=[os.path.join(lib_dir, "libltfat.a"),
                       os.path.join(lib_dir, "libltfatf.a")],
        )


######################
# Custom clean command
######################
class m_clean(clean):
    """ Remove build directories, and compiled file in the source tree"""

    def run(self):
        clean.run(self)
        if os.path.exists('build'):
            shutil.rmtree('build')
        if os.path.exists('doc' + os.path.sep + '_build'):
            shutil.rmtree('doc' + os.path.sep + '_build')
        for dirpath, dirnames, filenames in os.walk('.'):
            for filename in filenames:
                if (filename.endswith('.so') or
                        filename.endswith('.pyd') or
                        filename.endswith('.dll') or
                        filename.endswith('.pyc')):
                    os.unlink(os.path.join(dirpath, filename))
            for dirname in dirnames:
                if dirname == '__pycache__':
                    shutil.rmtree(os.path.join(dirpath, dirname))


##############################
# Custom sdist command
##############################
class m_sdist(sdist):
    """ Build source package

    WARNING : The stamping must be done on an default utf8 machine !
    """

    def run(self):
        if USE_COPYRIGHT:
            writeStamp()
            sdist.run(self)
            # eraseStamp()
        else:
            sdist.run(self)


####################
# Setup method
####################
def setup_package():
    """ Setup function"""
    # set version
    clean_stale_cmake_cache()
    VERSION = get_version()
    path_includes = ['.', './ltfatpy/comp']

    lib_dir = os.path.join('ltfat_C_kernel', 'lib')
    ltfat_dir = 'ltfatpy'
    set_version(ltfat_dir, get_version())
    # get the list of extensions
    extNames = findpyxfiles(ltfat_dir)
    # authors=[{"name" : "Denis Arrivault" },
    #              {"name" : "Florent Jaillet" },
    #              {"name" : "Valentin Emiya" , "email" : "valentin.emiya@lis-lab.fr"}],
    # and build up the set of Extension objects
    if USE_CYTHON:
        fileExt = ".pyx"
        print("Cython Used")
    else:
        fileExt = ".c"
        print("witout Cython")
    include_package_data = True
    extras_require = {
        'test': ['pytest', 'nose', 'coverage'],
        'doc': ["sphinx==5.0", "numpydoc", "sphinx_gallery", "matplotlib", "sphinx_rtd_theme"]}

    # 'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',

    extensions = [makeExtension(name, fileExt, lib_dir, path_includes) for name in extNames]

    setup(name="ltfatpy",
          version=VERSION,
          long_description=(read('README.rst') + '\n\n' + 
                            read('HISTORY.rst') + '\n\n' + 
                            read('AUTHORS.rst')),
          package_data={'ltfatpy.signals': ['*.wav'],
                        'ltfatpy.comp': ['*.pxd'],
                        'ltfatpy.tests.datasets': ['*.mat']},
          ext_modules=extensions,
          compiler_directives={'language_level': "3"},
          test_suite='nose.collector',
          include_package_data=include_package_data,
          extras_require=extras_require,
          tests_require=['nose', 'coverage'],
          cmdclass={'build_ext': m_build_ext,
                    'clean': m_clean, 'sdist': m_sdist},
             )


if __name__ == "__main__":
    setup_package()



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


""" Module of Kaiser-Bessel window calculation

Ported from ltfat_2.1.0/sigproc/firkaiser.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np

from ltfatpy.sigproc.normalize import normalize


def firkaiser(L, beta, centering='wp', norm='null'):
    """Kaiser-Bessel window

    - Usage:

        | ``g = firkaiser(L, beta)``
        | ``g = firkaiser(L, beta,...)``

    - Input parameters:

    :param int L: length of the window
    :param int beta: beta parameter of the Kaiser-Bessel window
    :param str centering: Flag specifying if the generated window **g** is
        whole point even when ``centering='wp'`` or half point even when
        ``centering='hp'``
    :param str norm: Normalization flag for the output. Please see the
        parameter **norm** in the help of
        :func:`~ltfatpy.sigproc.normalize.normalize` for possible values.

    - Output parameters:

    :returns: Kaiser-Bessel window of length ``L`` with parameter ``beta``
    :rtype: numpy.ndarray

    ``firkaiser(L, beta)`` computes the Kaiser-Bessel window of length **L**
    with parameter **beta**. The smallest element of the window is set to zero
    when the window has an even length. This gives the window perfect
    whole-point even symmetry, and makes it possible to use the window for a
    Wilson basis.

    .. note::

        This Python port only implements the ``'normal'`` option of the
        original ``firkaiser`` function available in LTFAT for Octave.

        The ``'derived'`` option is not currently implemented in this port
        because the original ``firkaiser`` function contains a reported
        `bug <http://sourceforge.net/p/ltfat/bugs/124/>`_.

        We are thus waiting for the correction of the original code to add the
        ``'derived'`` option to this port.

    .. seealso:: :func:`~ltfatpy.sigproc.firwin.firwin`
                 :func:`~ltfatpy.sigproc.normalize.normalize`

    - References:
        :cite:`opsc89`
    """

    """ NOTE:
    All the elements for the implementation of the stype='derived' option
    have been left as comments in the following code, so that it can be easily
    implemented once the corrected code will be provided by the LTFAT for
    Octave project.

    The function definition should be:
    def firkaiser(L, beta, centering='wp', stype='normal', norm='null'):

    The following should be added in the docstring:
        :param str stype: Flag specifying the type of generated window **g**,
        returning a normal Kaiser-Bessel window when ``stype='normal'`` and a
        derived Kaiser-Bessel window when ``stype='derived'``

    The test on stype should uncommented below, and the code for the 'derived'
    case should be uncommented and corrected.

    test_firkaiser, the test function of firkaiser should also be updated.
    """

    if not isinstance(beta, float):
        raise TypeError('beta must be a float.')
    cent = 0.
    if centering == 'hp':
        cent = 0.5

    # if stype == 'normal':
    if L == 1:
        g = np.array([1.])
    else:
        k = np.arange(L) + (L % 2)/2. - 0.5 + cent
        # NOTE: to take the sqrt of negative values we need to convert
        # the data to complex, otherwise np.sqrt returns nan
        tmp = k*(L-1.-k)
        tmp = tmp.astype('complex128')
        k = np.real(2. * beta / (L-1) * np.sqrt(tmp))
        g = np.i0(k) / np.i0(beta)

    g = np.fft.ifftshift(g)

    if ((centering == 'wp' and L % 2 == 0) or
            (centering == 'hp' and L % 2 == 1)):
        # Explicitly zero last element. This is done to get the right
        # symmetry, and because that element sometimes turn negative.
        g[int(np.floor(L/2.))] = 0.

    """
    elif stype == 'derived':
        if L % 2 == 1:
            raise ValueError('The length of the choosen window must be even.')

        if centering == 'wp':
            if L % 4 == 0:
                L2 = L/2 + 2
            else:
                L2 = L/2 + 1
        else:
            L2 = int(np.floor((L+1.)/2.))

        # Compute a normal Kaiser window
        g_normal = np.fft.fftshift(firkaiser(L2, beta, centering=centering))

        g1 = np.sqrt(np.cumsum(g_normal[:L2])/np.sum(g_normal[:L2]))

        if centering == 'wp':
            # NOTE: Here the Octave code differentiate two cases if L is even
            # or odd. This is useless, since this part of the code can only be
            # reached when L is even due to the error thrown above when L is
            # odd. So this Python port only includes the L even case.
            g = np.concatenate((g1[::-1], g1[1:L/2]))
        else:
            g = np.concatenate((g1[::-1], g1))

        if ((centering == 'wp' and L % 2 == 0) or
                (centering == 'hp' and L % 2 == 1)):
            # Explicitly zero last element. This is done to get the right
            # symmetry, and because that element sometimes turn negative.
            g[np.floor(L/2.)] = 0.
    """

    # In Octave/MATLAB, the besseli computation sometimes generates a zero
    # imaginary component. It is not sure if numpy.i0 does the same, but for
    # safety we keep this line.
    g = np.real(g)

    g = normalize(g, norm=norm)[0]
    return g



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module of dgtlength calculation

Ported from ltfat_2.1.0/gabor/dgtlength.m

.. moduleauthor:: Denis Arrivault
"""

import numpy as np

from ltfatpy.tools.lcm import lcm


def dgtlength(Ls, a, M):
    """DGT length from signal

    - Usage:

         | ``L = dgtlength(Ls, a, M)``

    - Input parameters:

    :param int Ls: Signal length
    :param int a: Length of time shift.
    :param int M: Number of channels.

    - Output parameters:

    :return: Corrected signal length
    :rtype: int

    ``dgtlength(Ls, a, M)`` returns the length of a Gabor system that is long
    enough to expand a signal of length **Ls**. Please see the help on
    :func:`~ltfatpy.gabor.dgt.dgt` for an explanation of the parameters **a**
    and **M**.

    .. seealso:: :func:`~ltfatpy.gabor.dgt.dgt`
    """
    if not(isinstance(M, int)):
        raise TypeError('M must be an integer')
    if not(isinstance(a, int)):
        raise TypeError('a must be an integer')
    if M <= 0:
        raise ValueError('M must be positive')
    if a <= 0:
        raise ValueError('a must be positive')
    if not(isinstance(Ls, (int, float, complex))):
        raise TypeError('Ls must be a scalar')

    Lsmallest = lcm(a, M)
    L = np.ceil(Ls/Lsmallest)*Lsmallest
    return int(L)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module of dgt coefficients plotting

Ported from ltfat_2.1.0/gabor/plotdgt.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np

from ltfatpy.gabor.tfplot import tfplot


def plotdgt(coef, a, **kwargs):
    """Plot dgt coefficients

    - Input parameters:

    :param numpy.ndarray coef: Gabor coefficients
    :param int a: Length of time shift used when computing **coef**
    :param `**kwargs`: ``plotdgt`` supports all the optional parameters of
        :func:`~ltfatpy.gabor.tfplot.tfplot`. Please see the help of
        :func:`~ltfatpy.gabor.tfplot.tfplot` for an exhaustive list.

    - Output parameters:

    :returns: The processed image data used in the plotting. Inputting this
        data directly to :func:`~matplotlib.pyplot.matshow` or similar
        functions will create the plot. This is useful for custom
        post-processing of the image data.
    :rtype: numpy.ndarray

    ``plotdgt(coef, a)`` plots the Gabor coefficients **coef**. The
    coefficients must have been produced with a time shift of **a**.

    The figure generated by this function places the zero-frequency in
    the center of the y-axis, with positive frequencies above and
    negative frequencies below.

    .. seealso:: :func:`~ltfatpy.gabor.dgt.dgt`,
        :func:`~ltfatpy.gabor.tfplot.tfplot`,
        :func:`~ltfatpy.gabor.sgram.sgram`,
        :func:`~ltfatpy.gabor.plotdgtreal.plotdgtreal`

    """

    M = coef.shape[0]

    # Move zero frequency to the center and Nyquist frequency to the top.
    if M % 2 == 0:
        coef = np.roll(coef, M//2-1, axis=0)
        yr = np.array([-1.+2./M, 1.])
    else:
        coef = np.roll(coef, (M-1)//2, axis=0)
        yr = np.array([-1.+2./M, 1.-2./M])

    coef = tfplot(coef, a, yr, **kwargs)
    return coef



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module of dgtreal calculation

Ported from ltfat_2.1.0/gabor/dgtreal.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import numpy as np

from ltfatpy.comp.gabpars_from_windowsignal import gabpars_from_windowsignal
from ltfatpy.comp.comp_sepdgtreal import comp_sepdgtreal


def dgtreal(f, g, a, M, L=None, pt='freqinv'):
    """Discrete Gabor transform for real signals

    - Usage:
        | ``(c, Ls, g) = dgtreal(f, g, a, M)``
        | ``(c, Ls, g) = dgtreal(f, g, a, M, L)``
        | ``(c, Ls, g) = dgtreal(f, g, a, M, L, pt)``

    - Input parameters:

    :param numpy.ndarray f: Input data. **f** dtype should be float64
    :param g: Window function.
    :param int a: Length of time shift.
    :param int M: Number of modulations.
    :param int L: Length of transform to do. Default is None.
    :param str pt: 'freqinv' or 'timeinv'. Default is 'freqinv'.
    :type g: str, dict or numpy.ndarray of float64

    - Output parameters:

    :returns: ``(c, Ls, g)``
    :rtype: tuple
    :var numpy.ndarray c: :math:`M*N` array of complex128 coefficients.
    :var int Ls: length of input signal
    :var numpy.ndarray g: updated window function. dtype is float64.

    ``dgtreal(f, g, a, M)`` computes the Gabor coefficients (also known as a
    windowed Fourier transform) of the real-valued input signal **f** with
    respect to the real-valued window **g** and parameters **a** and **M**.
    The output is a vector/matrix in a rectangular layout.

    As opposed to :func:`~ltfatpy.gabor.dgt.dgt` only the coefficients of the
    positive frequencies of the output are returned. ``dgtreal`` will refuse
    to work for complex valued input signals.

    The length of the transform will be the smallest multiple of **a** and
    **M** that is larger than the signal. **f** will be zero-extended to the
    length of the transform. If **f** is a matrix, the transformation is
    applied to each column. The length of the transform done can be obtained
    by ``L = c.shape[1] * a``.

    The window **g** may be a vector of numerical values, a text string or a
    dictionary. See the help of :py:meth:`~ltfatpy.gabor.gabwin` for more
    details.

    ``dgtreal(f, g, a, M, L)`` computes the Gabor coefficients as above, but
    does a transform of length **L**. **f** will be cut or zero-extended to
    length **L** before the transform is done.

    The ``dgtreal`` function  returns the length of the input signal **f**.
    This is handy for reconstruction:

    >>> (c, Ls, g) = dgtreal(f, g, a, M) # doctest: +SKIP
    >>> fr = idgtreal(c, gd, a, M, Ls) # doctest: +SKIP

    will reconstruct the signal **f** no matter what the length of **f** is,
    provided that **gd** is a dual window of **g**.

    It also outputs the window used in the transform. This is useful if the
    window was generated from a description in a string or dictionary.

    See the help on :func:`~ltfatpy.gabor.dgt.dgt` for the definition of the
    discrete Gabor transform. This routine will return the coefficients for
    channel frequencies from 0 to ``floor(M/2)``.

    ``dgtreal`` optionnaly takes a **pt** argument:

        'freqinv'
            Compute a ``dgtreal`` using a frequency-invariant phase. This
            is the default convention described in the help for
            :func:`~ltfatpy.gabor.dgt.dgt`.
        'timeinv'
            Compute a ``dgtreal`` using a time-invariant phase. This
            convention is typically used in filter bank algorithms.

    ``dgtreal`` can be used to manually compute a spectrogram, if you
    want full control over the parameters and want to capture the output.

    - Example:

    >>> import matplotlib.pyplot as plt
    >>> from ltfatpy import greasy
    >>> (f,fs) = greasy()   # Input test signal
    >>> a = 10         # Downsampling factor in time
    >>> M = 200        # Total number of channels, only 101 will be computed
    >>> # Compute the coefficients using a 20 ms long Hann window
    >>> c = dgtreal(f, {'name' : 'hann', 'M' : 0.02*fs}, a, M)[0]
    >>> # Visualize the coefficients as a spectrogram
    >>> dynrange = 90  # 90 dB dynamical range for the plotting
    >>> from ltfatpy import plotdgtreal
    >>> coef = plotdgtreal(c, a, M, fs=fs, dynrange=dynrange)
    >>> plt.show()

    .. image:: images/dgtreal.png
       :width: 700px
       :alt: spectrogram image
       :align: center

    .. seealso::  :func:`~ltfatpy.gabor.dgt.dgt`,
        :func:`~ltfatpy.gabor.idgtreal.idgtreal`,
        :func:`~ltfatpy.gabor.gabwin.gabwin`, :func:`dwilt`,
        :func:`~ltfatpy.gabor.gabtight.gabtight`,
        :func:`~ltfatpy.gabor.plotdgtreal.plotdgtreal`

    - References:
        :cite:`fest98,gr01`
    """
    (f, gnum, _, Ls) = gabpars_from_windowsignal(f, g, a, M, L)[0:4]

    if not np.issubdtype(gnum.dtype, np.floating):
        raise ValueError('The window must be real-valued.')

    # verify pt
    if pt == 'timeinv':
        pt = 1
    elif pt == 'freqinv':
        pt = 0
    else:
        raise ValueError("pt argument should be 'timeinv' or 'freqinv'.")

    c = comp_sepdgtreal(f, gnum, a, M, pt)
    return (c, Ls, gnum)


if __name__ == '__main__':  # pragma: no cover
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the gabphasegrad function

.. moduleauthor:: Florent Jaillet
"""

import unittest
import numpy as np
from numpy.testing import assert_array_equal, assert_allclose

from ltfatpy.gabor.gabphasegrad import gabphasegrad
from ltfatpy.tests.datasets.read_ref_mat import read_ref_mat
from ltfatpy.tests.datasets.get_dataset_path import get_dataset_path

# NOTE: The reference values used in the tests correspond to results
# obtained with Octave using ltfat 2.1.0


class TestGabphasegrad(unittest.TestCase):

    # Called before the tests.
    def setUp(self):
        print('\nStart TestGabphasegrad')

    # Called after the tests.
    def tearDown(self):
        print('Test done')

    def test_exceptions(self):
        """Check that the right exceptions are raised when expected
        """
        # method should be a string containing either 'dgt' or 'phase' or 'abs'
        self.assertRaises(TypeError, gabphasegrad, 1)
        self.assertRaises(TypeError, gabphasegrad, 1.0)
        self.assertRaises(TypeError, gabphasegrad, (1, 2))
        self.assertRaises(ValueError, gabphasegrad, 'test')

        # check that passing a wrong L with method 'dgt' raises an exception
        f = np.random.random((8,))
        g = 'gauss'
        a = 2
        M = 4
        self.assertRaises(ValueError, gabphasegrad, 'dgt', f, g, a, M, L=3)

        # check that passing a too long window g with method 'dgt' raises an
        # exception
        g = np.ones(9)
        self.assertRaises(ValueError, gabphasegrad, 'dgt', f, g, a, M)
        g = np.ones(5)
        self.assertRaises(ValueError, gabphasegrad, 'dgt', f, g, a, M, L=4)

        # check that passing complex values instead of real phase values with
        # method 'phase' raises an exception
        shape = (3, 2)
        cphase = np.random.random(shape) + 1.0j*np.random.random(shape)
        self.assertRaises(TypeError, gabphasegrad, 'phase', cphase, a)

        # check that passing negative values when an absolute value is
        # expected with method 'abs' raises an exception
        s = - np.random.random(shape)
        g = 'gauss'
        self.assertRaises(ValueError, gabphasegrad, 'abs', s, g, a)

        # check that passing a non-gaussian window g with method 'abs' raises
        # an exception
        s = np.random.random(shape)
        g = np.ones(4)
        self.assertRaises(ValueError, gabphasegrad, 'abs', s, g, a)

    def test_known(self):
        """Check gabphasegrad on some known results taken from Octave
        """
        filename = get_dataset_path('gabphasegrad_ref.mat')
        data = read_ref_mat(filename)

        for inputs, outputs in data:
            method = str(inputs['method'])
            if method == 'dgt':
                f, g, a, M = (inputs[var] for var in ('f', 'g', 'a', 'M'))
                kwargs = dict()
                if 'L' in inputs:
                    kwargs['L'] = inputs['L']
                if 'minlvl' in inputs:
                    kwargs['minlvl'] = inputs['minlvl']
                if isinstance(g, str):
                    # force g to be str instead of unicode under Python 2
                    g = str(g)
                tgrad, fgrad, c = gabphasegrad(method, f, g, a, M, **kwargs)
                msg = 'wrong output in gabphasegrad with inputs ' + str(inputs)
                assert_allclose(tgrad, outputs[0], rtol=4e-15, err_msg=msg)
                assert_allclose(fgrad, outputs[1], rtol=1e-15, err_msg=msg)
                assert_allclose(c, outputs[2], rtol=1e-15, err_msg=msg)
            elif method == 'phase':
                cphase, a = (inputs[var] for var in ('cphase', 'a'))
                tgrad, fgrad = gabphasegrad(method, cphase, a)
                msg = 'wrong output in gabphasegrad with inputs ' + str(inputs)
                assert_array_equal(tgrad, outputs[0], msg)
                assert_array_equal(fgrad, outputs[1], msg)
            elif method == 'abs':
                s, g, a = (inputs[var] for var in ('s', 'g', 'a'))
                kwargs = dict()
                if 'difforder' in inputs:
                    kwargs['difforder'] = inputs['difforder']
                if isinstance(g, str):
                    # force g to be str instead of unicode under Python 2
                    g = str(g)
                tgrad, fgrad = gabphasegrad(method, s, g, a, **kwargs)
                msg = 'wrong output in gabphasegrad with inputs ' + str(inputs)
                assert_array_equal(tgrad, outputs[0], msg)
                assert_array_equal(fgrad, outputs[1], msg)


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGabphasegrad)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the read_dgt_signal_ex_mat function

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import unittest
import numpy
try:
    from math import gcd
except ImportError:
    # fractions.gcd() is deprecated since Python 3.5 and math.gcd() should be
    # used instead, but for backward compatibilty we use fractions.gcd() if
    # math.gcd() is not available
    from fractions import gcd

from ltfatpy.tests.datasets.read_dgt_signal_ex_mat import DgtSignals
from ltfatpy.tests.datasets.get_dataset_path import get_dataset_path


class TestReadDgtSignalExt(unittest.TestCase):
    """ Test read_dgt_signal_ex.py """

    # Called before the tests.
    def setUp(self):
        self.filename = get_dataset_path('sepdgt_signal_ex.mat')

    # Called after the tests.
    def tearDown(self):
        print('Test done')

    def test_default(self):
        dgs = DgtSignals(self.filename)
        (TYPE, PHASETYPE, L, W, a, M, gl, SIGNAL, WINDOW, DGT, DUAL_WINDOW,
         IDGT) = dgs.read_next_signal()
        while TYPE != '':
            '''
            print ("\n\nTYPE = ", TYPE, "\nPHASETYPE = ", PHASETYPE, "\nL = ",
                   L, "\nW = ", W, "\na = ", a, "\nM = ", M, "\ngl = ",
                   gl, "\nSIGNAL =\n", SIGNAL, "\nWINDOW =\n", WINDOW,
                   "\nDGT\n", DGT, "\nDUAL_WINDOW\n", DUAL_WINDOW,
                   "\nIDGT\n", IDGT)
            '''

            assert (TYPE == "REAL" or TYPE == "CMPLX")
            assert (PHASETYPE == "freqinv" or PHASETYPE == "timeinv")
            assert isinstance(L, int)
            assert isinstance(W, int)
            assert isinstance(a, int)
            assert isinstance(M, int)
            assert isinstance(gl, int)
            assert type(SIGNAL).__module__ == numpy.__name__
            assert type(WINDOW).__module__ == numpy.__name__
            assert type(DGT).__module__ == numpy.__name__
            assert type(IDGT).__module__ == numpy.__name__
            assert WINDOW.size == gl
            if (gl == M):
                assert DUAL_WINDOW.size == gl
            elif (a != M):
                assert DUAL_WINDOW.size == a * M // gcd(a, M)
            else:
                assert DUAL_WINDOW.size == a * 2

            if W == 1:
                assert SIGNAL.size == L
                assert IDGT.size == L
            else:
                assert SIGNAL.shape == (L, W)
                assert IDGT.shape == (L, W)

            if TYPE == "REAL":
                if W == 1:
                    assert DGT.shape == ((M//2+1), L//a)
                else:
                    assert DGT.shape == ((M//2+1), L//a, W)
            else:
                if W == 1:
                    assert DGT.shape == (M, L//a)
                else:
                    assert DGT.shape == (M, L//a, W)
            (TYPE, PHASETYPE, L, W, a, M, gl, SIGNAL, WINDOW, DGT, DUAL_WINDOW,
             IDGT) = dgs.read_next_signal()

if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestReadDgtSignalExt)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the gabimagepars function

NOTE: Only the non-graphical features of gabimagepars are tested here

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import unittest

from ltfatpy.gabor.gabimagepars import gabimagepars

# NOTE: The reference values used in the tests correspond to results
# obtained with Octave using ltfat 2.1.0


class TestGabimagepars(unittest.TestCase):

    # Called before the tests.
    def setUp(self):
        print('\nStart TestGabimagepars')

    # Called after the tests.
    def tearDown(self):
        print('Test done')

    def test_working(self):
        """Check that gabimagepars is working as expected
        """
        in_val = ((10, 11, 12), (10, 5, 5), (100, 60, 18), (512, 64, 32))
        out_oct = ((1, 10, 10, 10, 10), (2, 5, 10, 5, 5), (3, 18, 108, 36, 34),
                   (8, 32, 512, 64, 64))
        inputs = {}
        for val, out_ref in zip(in_val, out_oct):
            inputs['Ls'] = val[0]
            inputs['x'] = val[1]
            inputs['y'] = val[2]
            out = gabimagepars(**inputs)
            msg = ('wrong output in gabimagepars with inputs ' + str(inputs))
            self.assertEqual(out_ref, out, msg)

        self.assertRaises(ValueError, gabimagepars, 100, 6, 13)


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGabimagepars)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module of Gabor window calculation

Ported from ltfat_2.1.0/gabor/gabwin.m


.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

from ltfatpy.comp.comp_window import comp_window


def gabwin(g, a, M, L=None):
    """Computes a Gabor window

    - Usage:

        | ``(g, info) = gabwin(g, a, M, L)``

    - Input parameters:

    :param g: the gabor window
    :param int a: the length of time shift
    :param int M: the number of channels
    :param int L: the length of window (optional)
    :type g: numpy.ndarray or str or dict

    - Output parameters:

    :returns: ``(g, info)``
    :rtype: tuple
    :var numpy.ndarray g: the computed gabor window
    :var dict info: the information dictionary (see before)

    With the transform length parameter **L** specified, it computes a window
    that fits well with the specified number of channels **M**, time shift
    **a** and transform length **L**. The window itself is specified by a
    text description or a dictionary containing additional parameters.

    The window can be specified directly as a vector of numerical
    values. In this case, ``gabwin`` only checks assumptions about transform
    sizes etc.

    Without the transform length parameter **L**, it does the same, but the
    window must be a FIR window.

    The window can be specified as one of the following text strings:

        ===========  =======================================
        'gauss'      Gaussian window fitted to the lattice,
                     i.e. :math:`tfr = a * M / L`.

        'dualgauss'  Canonical dual of Gaussian window.

        'tight'      Tight window generated from a Gaussian.
        ===========  =======================================

    In these cases, a long window is generated with a length of **L**.

    It is also possible to specify one of the window names from
    :func:`~ltfatpy.sigproc.firwin.firwin`. In such a case,
    :func:`~ltfatpy.gabor.gabwin.gabwin` will generate the specified FIR
    window with a length of **M**.

    The window can also be specified as a dictionary. The possibilities are:

        - {'name': 'gauss', 'tfr': 1.0, 'fs': 0.0, 'width': 0.0,
           'bw': 0.0, 'c_f': 0.0, 'centering': 'hp', 'delay': 0.0, 'norm': 2}:
           Additional parameters are passed to
           :func:`~ltfatpy.fourier.pgauss.pgauss`.
        - {'name': ('dual',...), ...}:
            Canonical dual window of whatever follows. See the examples below.
        - {'name': ('tight',...), ...}:
            Canonical tight window of whatever follows.

    It is also possible to specify one of the window names from
    :func:`~ltfatpy.sigproc.firwin.firwin` as a string. In this case, the
    remaining entries of the cell array are passed directly to
    :func:`~ltfatpy.sigproc.firwin.firwin`.

    - Some examples:

        To compute a Gaussian window of length **L** fitted for a system with
        time-shift **a** and **M** channels use::

            g = gabwin('gauss', a, M, L)[0]

        To compute Gaussian window with equal time and frequency support
        irrespective of **a** and **M**::

            g = gabwin({'name': 'gauss', 'tfr': 1}, a, M, L)[0]

        To compute the canonical dual of a Gaussian window fitted for a
        system with time-shift **a** and **M** channels::

            gd = gabwin('gaussdual', a, M, L)[0]

        To compute the canonical tight window of the Gaussian window fitted
        for the system::

            gd = gabwin({'name': ('tight','gauss')}, a, M, L)[0]

        To compute the dual of a Hann window of length 20::

            g = gabwin({'name': ('dual', 'hann'), 'M': 20}, a, M, L)[0]

    The returned **info** dictionary provides some information about the
    computed window:

    =========   ===========================================================
    keys        Values
    =========   ===========================================================
    'gauss'     True if the window is a Gaussian.
    'tfr'       Time/frequency support ratio of the window.
                Set whenever it makes sense.
    'wasrow'    True if input was a row window
    'isfir'     True if input is an FIR window
    'isdual'    True if output is the dual window of the auxiliary window.
    'istight'   True if output is known to be a tight window.
    'auxinfo'   Info about auxiliary window.
    'gl'        Length of window.
    =========   ===========================================================

    .. seealso:: :func:`~ltfatpy.fourier.pgauss.pgauss`,
        :func:`~ltfatpy.sigproc.firwin.firwin`, :func:`wilwin`
    """
    # Assert correct input.
    (g, info) = comp_window(g, a, M, L)
#     if info['isfir']:
#         if info['istight']:
#             # g = g / math.sqrt(2)
    return (g, info)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the gabwin function

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import unittest
import random
import numpy as np

# from math import *
# import matplotlib.pyplot as plt
# from fractions import gcd

from ltfatpy.gabor.gabwin import gabwin
from ltfatpy.fourier.pgauss import pgauss
from ltfatpy.fourier.psech import psech
from ltfatpy.gabor.gabdual import gabdual
from ltfatpy.gabor.gabtight import gabtight
from ltfatpy.sigproc.firwin import firwin
from ltfatpy.gabor.dgtlength import dgtlength
from ltfatpy.comp.arg_firwin import arg_firwin
from ltfatpy.fourier import isevenfunction as ie
from ltfatpy.sigproc.fir2long import fir2long
from ltfatpy.tools.lcm import lcm


class TestGabWin(unittest.TestCase):
    # Called before the tests.
    def setUp(self):
        pass

    # Called after the tests.
    def tearDown(self):
        print('Test done')

    def test_default(self):
        a = 10
        M = 40
        self.assertRaises(ValueError, gabwin, "gauss", a, M)
        L = 160
        mess = "a = {0:d}, M = {1:d}, L = {2:d}".format(a, M, L)
        (g, info) = gabwin("gauss", a, M, L)
        mess += "\ng = " + str(g)
        mess += "\ninfo = " + str(info)
        self.assertTrue(info['gauss'], mess)
        self.assertTrue(info['wasreal'], mess)
        self.assertFalse(info['istight'], mess)
        self.assertFalse(info['isdual'], mess)
        self.assertFalse(info['wasrow'], mess)
        self.assertFalse(info['isfir'], mess)
        self.assertEqual(info["tfr"], a*M/L, mess)
        self.assertEqual(info["gl"], len(g), mess)
        gt = pgauss(L, a*M/L)[0]
        mess += "\ngt = " + str(gt)
        np.testing.assert_array_almost_equal(g, gt, 10, mess)

    def test_str_entries(self):
        a = random.randint(10, 40)
        M = random.randint(10, 40)
        L = M * random.randint(2, 5)

        # psech
        mess = "a = {0:d}, M = {1:d}, L = {2:d}".format(a, M, L)
        (g, info) = gabwin("psech", a, M, L)
        mess += "\ng = " + str(g)
        mess += "\ninfo = " + str(info)
        self.assertFalse(info['gauss'], mess)
        self.assertTrue(info['wasreal'], mess)
        self.assertFalse(info['istight'], mess)
        self.assertFalse(info['isdual'], mess)
        self.assertFalse(info['wasrow'], mess)
        self.assertFalse(info['isfir'], mess)
        self.assertEqual(info["tfr"], a*M/L, mess)
        self.assertEqual(info["gl"], len(g), mess)
        gt = psech(L, a*M/L)[0]
        mess += "\ngt = " + str(gt)
        np.testing.assert_array_almost_equal(g, gt, 10, mess)
        self.assertRaises(ValueError, gabwin, "sech", a, M)

        # dualgauss
        mess = "a = {0:d}, M = {1:d}, L = {2:d}".format(a, M, L)
        (g, info) = gabwin("dualgauss", a, M, L)
        mess += "\ng = " + str(g)
        mess += "\ninfo = " + str(info)
        self.assertFalse(info['gauss'], mess)
        self.assertTrue(info['wasreal'], mess)
        self.assertFalse(info['istight'], mess)
        self.assertTrue(info['isdual'], mess)
        self.assertFalse(info['wasrow'], mess)
        self.assertFalse(info['isfir'], mess)
        self.assertEqual(info["tfr"], a*M/L, mess)
        self.assertEqual(info["gl"], len(g), mess)
        gt = gabdual(pgauss(L, a*M/L)[0], a, M)
        mess += "\ngt = " + str(gt)
        np.testing.assert_array_almost_equal(g, gt, 10, mess)
        self.assertRaises(ValueError, gabwin, "gaussdual", a, M)

        # tight
        L = dgtlength(L, a, M)
        mess = "a = {0:d}, M = {1:d}, L = {2:d}".format(a, M, L)
        (g, info) = gabwin("tight", a, M, L)
        mess += "\ng = " + str(g)
        mess += "\ninfo = " + str(info)
        self.assertFalse(info['gauss'], mess)
        self.assertTrue(info['wasreal'], mess)
        self.assertTrue(info['istight'], mess)
        self.assertFalse(info['isdual'], mess)
        self.assertFalse(info['wasrow'], mess)
        self.assertFalse(info['isfir'], mess)
        self.assertEqual(info["tfr"], a*M/L, mess)
        self.assertEqual(info["gl"], len(g), mess)
        gt = gabtight(None, a, M, L)
        mess += "\ngt = " + str(gt)
        np.testing.assert_array_almost_equal(g, gt, 10, mess)
        self.assertRaises(ValueError, gabwin, "tight", a, M)

        # firwin
        name = random.sample(list(arg_firwin()), 1)[0]
        mess = "a = {0:d}, M = {1:d}, L = {2:d}, name = {3:s}".format(a, M, L,
                                                                      name)
        (g, info) = gabwin(name, a, M, L)
        mess += "\ng = " + str(g)
        mess += "\ninfo = " + str(info)
        self.assertFalse(info['gauss'], mess)
        self.assertTrue(info['wasreal'], mess)
        if name in {'sine', 'cosine', 'sqrthann', 'sqrttria', 'itersine',
                    'ogg'}:
            self.assertTrue(info['istight'], mess)
        else:
            self.assertFalse(info['istight'], mess)
        self.assertFalse(info['isdual'], mess)
        self.assertFalse(info['wasrow'], mess)
        self.assertTrue(info['isfir'], mess)
        self.assertEqual(info["gl"], len(g), mess)
        gt = firwin(name, M, norm='2')[0]
        mess += "\ngt = " + str(gt)
        np.testing.assert_array_almost_equal(g, gt, 10, mess)

        # Unknown
        self.assertRaises(ValueError, gabwin, "foo", a, M, L)

    def test_simple_dictionnaries_entries(self):
        a = random.randint(10, 40)
        M = random.randint(10, 40)
        L = M * random.randint(2, 5)

        # default
        self.assertRaises(ValueError, gabwin, {'a': 1}, a, M, L)

        # pgauss without tfr
        mess = "a = {0:d}, M = {1:d}, L = {2:d}".format(a, M, L)
        (g, info) = gabwin({'name': 'pgauss'}, a, M, L)
        mess += "\ng = " + str(g)
        mess += "\ninfo = " + str(info)
        self.assertTrue(info['gauss'], mess)
        self.assertTrue(info['wasreal'], mess)
        self.assertFalse(info['istight'], mess)
        self.assertFalse(info['isdual'], mess)
        self.assertFalse(info['wasrow'], mess)
        self.assertFalse(info['isfir'], mess)
        self.assertEqual(info["tfr"], a*M/L, mess)
        self.assertEqual(info["gl"], len(g), mess)
        gt = pgauss(L, a*M/L)[0]
        mess += "\ngt = " + str(gt)
        np.testing.assert_array_almost_equal(g, gt, 10, mess)
        self.assertRaises(ValueError, gabwin, {'name': 'pgauss'}, a, M)
        # pgauss with tfr specified
        (g, info) = gabwin({'name': 'pgauss', 'tfr': 1}, a, M, L)
        mess = "a = {0:d}, M = {1:d}, L = {2:d}".format(a, M, L)
        mess += "\ng = " + str(g)
        mess += "\ninfo = " + str(info)
        self.assertEqual(info["tfr"], 1, mess)
        gt = pgauss(L)[0]
        mess += "\ngt = " + str(gt)
        np.testing.assert_array_almost_equal(g, gt, 10, mess)

        # psech without tfr
        mess = "a = {0:d}, M = {1:d}, L = {2:d}".format(a, M, L)
        (g, info) = gabwin({'name': 'psech'}, a, M, L)
        mess += "\ng = " + str(g)
        mess += "\ninfo = " + str(info)
        self.assertFalse(info['gauss'], mess)
        self.assertTrue(info['wasreal'], mess)
        self.assertFalse(info['istight'], mess)
        self.assertFalse(info['isdual'], mess)
        self.assertFalse(info['wasrow'], mess)
        self.assertFalse(info['isfir'], mess)
        self.assertEqual(info["tfr"], a*M/L, mess)
        self.assertEqual(info["gl"], len(g), mess)
        gt = psech(L, a*M/L)[0]
        mess += "\ngt = " + str(gt)
        np.testing.assert_array_almost_equal(g, gt, 10, mess)
        self.assertRaises(ValueError, gabwin, {'name': 'sech'}, a, M)
        # psech with tfr specified
        (g, info) = gabwin({'name': 'sech', 'tfr': 1}, a, M, L)
        mess = "a = {0:d}, M = {1:d}, L = {2:d}".format(a, M, L)
        mess += "\ng = " + str(g)
        mess += "\ninfo = " + str(info)
        self.assertEqual(info["tfr"], 1, mess)
        gt = psech(L)[0]
        mess += "\ngt = " + str(gt)
        np.testing.assert_array_almost_equal(g, gt, 10, mess)

        # Test adding args in dict
        name = 'gauss'
        mess = "a = {0:d}, M = {1:d}, L = {2:d}, name = {3:s}".format(a, M, L,
                                                                      name)
        (g, info) = gabwin({'name': name, 'centering': 'hp'}, a, M, L)
        mess += "\ng = " + str(g)
        mess += "\ninfo = " + str(info)
        self.assertFalse(ie.isevenfunction(g, centering='wp'), mess)

        # firwin without M
        name = random.sample(list(arg_firwin()), 1)[0]
        mess = "a = {0:d}, M = {1:d}, L = {2:d}, name = {3:s}".format(a, M, L,
                                                                      name)

        self.assertRaises(ValueError, gabwin, {'name': name}, a, M, L)

        # firwin with M
        name = random.sample(list(arg_firwin()), 1)[0]
        mess = "a = {0:d}, M = {1:d}, L = {2:d}, name = {3:s}".format(a, M, L,
                                                                      name)
        Ml = 2*M+1
        (gl, info) = gabwin({'name': name, 'M': Ml}, a, M)
        mess += "\nMl = " + str(Ml)
        mess += "\ng = " + str(gl)
        mess += "\ninfo = " + str(info)
        self.assertFalse(info['gauss'], mess)
        self.assertTrue(info['wasreal'], mess)
        if name in {'sine', 'cosine', 'sqrthann', 'sqrttria', 'itersine',
                    'ogg'}:
            self.assertTrue(info['istight'], mess)
        else:
            self.assertFalse(info['istight'], mess)
        self.assertFalse(info['isdual'], mess)
        self.assertFalse(info['wasrow'], mess)
        self.assertTrue(info['isfir'], mess)
        self.assertEqual(info["gl"], len(gl), mess)
        gt = firwin(name, Ml, norm='2')[0]
        gt = fir2long(gt, int(np.ceil(gt.shape[0] / M) * M))
        mess += "\ngt = " + str(gt)
        np.testing.assert_array_almost_equal(gl, gt, 10, mess)

        # Unknown
        self.assertRaises(ValueError, gabwin, {'name': 'foo'}, a, M, L)

    def test_composed_dictionnaries_entries(self):
        a = random.randint(10, 40)
        M = random.randint(10, 40)
        L = lcm(a, M)

        # default
        self.assertRaises(TypeError, gabwin, {'name': 1}, a, M, L)

        # dual psech
        tfr = 10
        gd = {'name': ('dual', 'sech'), 'tfr': tfr}
        mess = "a = {0:d}, M = {1:d}, L = {2:d}".format(a, M, L)
        mess += str(gd)
        (g, info) = gabwin(gd, a, M, L)
        mess += "\ng = " + str(g)
        mess += "\ninfo = " + str(info)
        self.assertFalse(info['gauss'], mess)
        self.assertTrue(info['wasreal'], mess)
        self.assertFalse(info['istight'], mess)
        self.assertTrue(info['isdual'], mess)
        self.assertFalse(info['wasrow'], mess)
        self.assertFalse(info['isfir'], mess)
        self.assertEqual(info['auxinfo']["tfr"], tfr, mess)
        self.assertEqual(info["gl"], len(g), mess)
        gt = gabdual(psech(L, tfr)[0], a, M, L)
        mess += "\ngt = " + str(gt)
        np.testing.assert_array_almost_equal(g, gt, 10, mess)
        self.assertRaises(ValueError, gabwin, gd, a, M)

        # dual pgauss
        tfr = 10
        gd = {'name': ('dual',), 'tfr': tfr}
        mess = "a = {0:d}, M = {1:d}, L = {2:d}".format(a, M, L)
        mess += str(gd)
        (g, info) = gabwin(gd, a, M, L)
        mess += "\ng = " + str(g)
        mess += "\ninfo = " + str(info)
        self.assertTrue(info['auxinfo']['gauss'], mess)
        self.assertTrue(info['wasreal'], mess)
        self.assertFalse(info['istight'], mess)
        self.assertTrue(info['isdual'], mess)
        self.assertFalse(info['wasrow'], mess)
        self.assertFalse(info['isfir'], mess)
        self.assertEqual(info['auxinfo']["tfr"], tfr, mess)
        self.assertEqual(info["gl"], len(g), mess)
        gt = gabdual(pgauss(L, tfr)[0], a, M, L)
        mess += "\ngt = " + str(gt)
        np.testing.assert_array_almost_equal(g, gt, 10, mess)
        self.assertRaises(ValueError, gabwin, gd, a, M)

        # dual fir
        name = random.sample(list(arg_firwin()), 1)[0]
        mess = "a = {0:d}, M = {1:d}, L = {2:d}, name = {3:s}".format(a, M, L,
                                                                      name)
        (g, info) = gabwin({'name': ('dual', name)}, a, M)
        mess += "\ng = " + str(g)
        mess += "\ninfo = " + str(info)
        self.assertFalse(info['gauss'], mess)
        self.assertTrue(info['wasreal'], mess)
        self.assertFalse(info['istight'], mess)
        self.assertTrue(info['isdual'], mess)
        self.assertFalse(info['wasrow'], mess)
        self.assertTrue(info['isfir'], mess)
        self.assertEqual(info["gl"], len(g), mess)
        gt = firwin(name, M, norm='2')[0]
        gt = gabdual(gt, a, M, L)
        mess += "\ngt = " + str(gt)
        np.testing.assert_array_almost_equal(g, gt, 10, mess)

        # tight fir
        name = random.sample(list(arg_firwin()), 1)[0]
        mess = "a = {0:d}, M = {1:d}, L = {2:d}, name = {3:s}".format(a, M, L,
                                                                      name)
        (g, info) = gabwin({'name': ('tight', name)}, a, M)
        mess += "\ng = " + str(g)
        mess += "\ninfo = " + str(info)
        self.assertFalse(info['gauss'], mess)
        self.assertTrue(info['wasreal'], mess)
        self.assertTrue(info['istight'], mess)
        self.assertFalse(info['isdual'], mess)
        self.assertFalse(info['wasrow'], mess)
        self.assertTrue(info['isfir'], mess)
        self.assertEqual(info["gl"], len(g), mess)
        gt = firwin(name, M, norm='2')[0]
        gt = gabtight(gt, a, M, L)
        mess += "\ngt = " + str(gt)
        np.testing.assert_array_almost_equal(g, gt, 10, mess)

    def test_array_entries(self):
        a = random.randint(10, 40)
        M = random.randint(10, 40)
        L = lcm(a, M)
        mess = "a = {0:d}, M = {1:d}, L = {2:d}".format(a, M, L)
        (g, info) = gabwin(np.reshape(pgauss(L)[0], (1, L)), a, M, L)
        mess += "\ng = " + str(g)
        mess += "\ninfo = " + str(info)
        self.assertTrue(info['wasreal'], mess)
        self.assertFalse(info['istight'], mess)
        self.assertFalse(info['isdual'], mess)
        self.assertTrue(info['wasrow'], mess)
        self.assertFalse(info['isfir'], mess)
        self.assertEqual(info["gl"], L, mess)
        gt = pgauss(L)[0]
        mess += "\ngt = " + str(gt)
        np.testing.assert_array_almost_equal(g, gt, 10, mess)

if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGabWin)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the lcm function

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import unittest
import numpy as np

from ltfatpy.tools.lcm import lcm


class TestLcm(unittest.TestCase):

    # Called before the tests.
    def setUp(self):
        print('\nStart TestLcm')

    # Called after the tests.
    def tearDown(self):
        print('Test done')

    def test_types(self):
        """Checking input and output types
        """
        # test with str input, when int is expected
        self.assertRaises(TypeError, lcm, 2, '1')
        # test with float input, when int is expected
        self.assertRaises(TypeError, lcm, 2., 1)
        # check output type (int expected)
        self.assertEqual(type(lcm(3, 5)), int, '\nOutput type should be int')

    def test_known(self):
        """Checking lcm on some known results taken from Octave
        """
        ref = ((30, 12, 60), (-20, 15, -60), (6, -9, -18), (-12, -9, 36),
               (0, 3, 0), (-12, 0, -0), (0, 0, 0))
        inputs = {}
        for X, Y, res in ref:
            inputs['X'] = X
            inputs['Y'] = Y
            msg = 'Wrong output of lcm with inputs ' + str(inputs)
            self.assertEqual(lcm(**inputs), res, msg)

    def test_symmetry(self):
        """Checking that lcm is symmetric
        """
        values = np.random.randint(-1000, 1000, (5,))
        for X in values:
            for Y in values:
                X = int(X)
                Y = int(Y)
                msg = ('lcm is not symetric with X = {0:d} and '
                       'Y = {1:d}'.format(X, Y))
                self.assertEqual(lcm(X, Y), lcm(Y, X), msg)

    def test_primes(self):
        """Checking lcm with prime numbers
        """
        primes = (2, 3, 5, 7, 11, 13, 17)
        inputs = {}
        for X in primes:
            for Y in primes:
                inputs['X'] = X
                inputs['Y'] = Y
                msg = 'Wrong output of lcm with prime inputs ' + str(inputs)
                if inputs['X'] == inputs['Y']:
                    self.assertEqual(lcm(**inputs), inputs['X'], msg)
                else:
                    self.assertEqual(lcm(**inputs),
                                     inputs['X']*inputs['Y'], msg)

if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestLcm)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the gspi function

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import unittest
from numpy.testing import assert_array_equal

from ltfatpy.signals.gspi import gspi
from ltfatpy.tests.datasets.read_ref_mat import read_ref_mat
from ltfatpy.tests.datasets.get_dataset_path import get_dataset_path


class TestGspi(unittest.TestCase):

    # Called before the tests.
    def setUp(self):
        print('\nStart TestGspi')

    # Called after the tests.
    def tearDown(self):
        print('Test done')

    def test_data(self):
        """ Checking that the read data match the expectations
        """
        filename = get_dataset_path('gspi_ref.mat')
        # Known values of samples as read by Octave with ltfat 2.1.0
        data = read_ref_mat(filename)
        inputs, outputs = data[0]
        (s, fs) = gspi()
        self.assertEqual(s.shape, outputs[0].shape,
                         'Wrong signal shape in gspi')
        assert_array_equal(s, outputs[0], 'Wrong sample values in gspi')
        self.assertEqual(fs, outputs[1], 'Wrong sampling frequency in gspi')


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGspi)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module of symmetrical zero-extension or cut of data

Ported from ltfat_2.1.0/fourier/middlepad.m

.. moduleauthor:: Florent Jaillet
"""

import numpy as np

from ltfatpy.comp.assert_sigreshape_pre import assert_sigreshape_pre
from ltfatpy.comp.assert_sigreshape_post import assert_sigreshape_post


def middlepad(f, L, dim=None, centering='wp'):
    """Symmetrically zero-extends or cuts a function

    - Usage:

        | ``h = middlepad(f, L)``
        | ``h = middlepad(f, L, dim)``
        | ``h = middlepad(f, L, ...)``

    - Input parameters:

    :param numpy.ndarray f: Input array
    :param int L: Length of the output array
    :param int dim: Axis over which to zero-extend or cut **f**
    :param str centering: Flag specifying if **f** is whole point even when
        ``centering='wp'`` or half point even when ``centering='hp'``

    - Output parameters:

    :returns: Zero-extended or cut array
    :rtype: numpy.ndarray

    ``middlepad(f, L)`` zero-extends or cuts **f** to length **L** by
    inserting zeros in the middle of the vector, or by cutting in the middle
    of the vector.

    If **f** is whole-point even, ``middlepad(f, L)`` will also be
    whole-point even.

    ``middlepad(f, L, dim)`` does the same along dimension **dim**.

    If **f** has even length, then **f** will not be purely zero-extended,
    but the last element will be repeated once and multiplied by ``1/2``.
    That is, the support of **f** will increase by one!

    Adding the flag ``centering='wp'`` will cut or extend whole point even
    functions (the default). Adding ``centering='hp'`` will do the same for
    half point even functions.

    .. seealso:: :func:`~ltfatpy.fourier.isevenfunction.isevenfunction`,
                 :func:`~ltfatpy.sigproc.fir2long.fir2long`,
                 :func:`~ltfatpy.fourier.fftresample.fftresample`
    """

    # Note: For a future improvement, it might be possible to replace the use
    # of numpy.concatenate with numpy.r_ in this function to simplify the code.

    if not isinstance(L, int):
        raise TypeError('L must be an integer.')

    if L < 1:
        raise ValueError('L must be larger than 0.')

    f, L, Ls, W, dim, permutedsize, order = assert_sigreshape_pre(f, L, dim)

    Lorig = Ls

    # Skip the main section if there is nothing to do. This is necessary
    # because some of the code below cannot handle the case of 'nothing to do'
    if L != Ls:
        if centering == 'wp':

            # ---------------   WPE case --------------------------------------
            if Lorig == 1:
                # Rather trivial case
                h = np.concatenate((f[np.newaxis, 0, :],
                                    np.zeros((L-1, W), dtype=f.dtype)))

            else:

                if Lorig > L:
                    # Cut

                    if L % 2 == 0:

                        # L even. Use average of endpoints.
                        h = np.concatenate((f[:L//2, :],
                                            (f[np.newaxis, L//2, :] +
                                             f[np.newaxis, Lorig-L//2, :]) / 2,
                                            f[Lorig-L//2+1:Lorig, :]))

                    else:

                        # No problem, just cut.
                        h = np.concatenate((f[:(L+1)//2, :],
                                            f[Lorig-(L-1)//2:Lorig, :]))

                else:
                    d = L - Lorig

                    # Extend
                    if Lorig % 2 == 0:
                        # Lorig even. We must split a value.
                        h = np.concatenate((f[:Lorig//2, :],
                                            f[np.newaxis, Lorig//2, :]/2,
                                            np.zeros((d-1, W), dtype=f.dtype),
                                            f[np.newaxis, Lorig//2, :]/2,
                                            f[Lorig//2+1:Lorig, :]))

                    else:
                        # Lorig is odd, we can just insert zeros.
                        h = np.concatenate((f[:(Lorig+1)//2, :],
                                            np.zeros((d, W), dtype=f.dtype),
                                            f[(Lorig+1)//2:Lorig, :]))

        elif centering == 'hp':

            # ------------------ HPE case ------------------------------------

            # NOTE: There is a bug here in LTFAT 2.1.0 for Octave, see:
            # https://sourceforge.net/p/ltfat/bugs/123
            # This bug arise because the case "if Lorig==1" hasn't been
            # implemented in the Octave code when centering = 'hp'.
            # To solve this bug, we simply remove the test "if Lorig==1", which
            # seems to lead to satisfactory results.

            if Lorig > L:

                d = Lorig-L
                # Cut

                if L % 2 == 0:
                    # L even

                    # No problem, just cut.
                    h = np.concatenate((f[:L//2, :], f[Lorig-L//2:Lorig, :]))

                else:
                    # Average of endpoints.
                    h = np.concatenate((f[:(L-1)//2, :],
                                        (f[np.newaxis, (L-1)//2, :] +
                                         f[np.newaxis, Lorig-(L+1)//2, :]) / 2,
                                        f[Lorig-(L-1)//2:Lorig, :]))

            else:

                d = L-Lorig

                # Extend
                if Lorig % 2 == 0:

                    # Lorig even. We can just insert zeros in the middle.
                    h = np.concatenate((f[:Lorig//2, :],
                                        np.zeros((d, W), dtype=f.dtype),
                                        f[Lorig//2:Lorig, :]))

                else:
                    # Lorig odd. We need to split a value in two
                    h = np.concatenate((f[:(Lorig-1)//2, :],
                                        f[np.newaxis, (Lorig-1)//2, :]/2,
                                        np.zeros((d-1, W), f.dtype),
                                        f[np.newaxis, (Lorig-1)//2, :]/2,
                                        f[(Lorig+1)//2:Lorig, :]))

    else:
        # we don't want this function to return a reference or a view to the
        # input, so we make a copy
        h = f.copy()

    h = assert_sigreshape_post(h, dim, permutedsize, order)

    return h



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the middlepad function

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import unittest
import numpy as np
from numpy.testing import assert_array_equal
from copy import deepcopy

from ltfatpy.fourier.middlepad import middlepad


class TestMiddlepad(unittest.TestCase):

    # Called before the tests.
    def setUp(self):
        print('\nStart TestMiddlepad')

    # Called after the tests.
    def tearDown(self):
        print('Test done')

    def test_exceptions(self):
        """Check that the right exceptions are raised when expected
        """
        f = np.random.random((3,))
        # L must be an integer, check that we get an error if not
        self.assertRaises(TypeError,  middlepad, f, 'test')
        self.assertRaises(TypeError,  middlepad, f, 3.4)

        # L must be larger than 0, check that we get an error if not
        self.assertRaises(ValueError,  middlepad, f, -2)

    def test_known(self):
        """Checking middlepad on some known results taken from Octave
        """
        inputs = {}
        N_val = (1, 1, 3, 4, 3, 4, 7, 8, 7, 8)
        L_val = (2, 3, 7, 7, 8, 8, 3, 3, 4, 4)
        centerings = ('wp', 'hp')

        # Values of h as returned in Octave with ltfat 2.1.0
        all_h_oct = ((np.array([1., 0.]),
                      np.array([1., 0., 0.]),
                      np.array([1., 2., 0., 0., 0., 0., 3.]),
                      np.array([1., 2., 1.5, 0., 0., 1.5, 4.]),
                      np.array([1., 2., 0., 0., 0., 0., 0., 3.]),
                      np.array([1., 2., 1.5, 0., 0., 0., 1.5, 4.]),
                      np.array([1., 2., 7.]),
                      np.array([1., 2., 8.]),
                      np.array([1., 2., 4.5, 7.]),
                      np.array([1., 2., 5., 8.])),
                     (np.array([0.5, 0.5]),
                      np.array([0.5, 0., 0.5]),
                      np.array([1., 1., 0., 0., 0., 1., 3.]),
                      np.array([1., 2., 0., 0., 0., 3., 4.]),
                      np.array([1., 1., 0., 0., 0., 0., 1., 3.]),
                      np.array([1., 2., 0., 0., 0., 0., 3., 4.]),
                      np.array([1., 4., 7.]),
                      np.array([1., 4.5, 8.]),
                      np.array([1., 2., 6., 7.]),
                      np.array([1., 2., 7., 8.])))

        for centering, h_oct in zip(centerings, all_h_oct):
            inputs['centering'] = centering
            for N, L, h_ref in zip(N_val, L_val, h_oct):
                inputs['L'] = L
                inputs['f'] = np.arange(1., float(N+1))
                h = middlepad(**inputs)
                msg = ('Wrong value in putput of middlepad with inputs ' +
                       str(inputs))
                assert_array_equal(h_ref, h, msg)

    def test_known_shapes(self):
        """Check that middlepad resturns the same shapes as in Octave
        """
        shapes_in = ((1, 1), (2, 1), (1, 2), (1, 2, 3), (1, 1, 2),
                     (1, 1, 2, 3))
        shapes_out = ((3, 1), (3, 1), (1, 3), (3, 2, 3), (1, 1, 3),
                      (3, 1, 2, 3))
        inputs = {}
        inputs['L'] = 3
        for shape_in, shape_out in zip(shapes_in, shapes_out):
            inputs['f'] = np.ones(shape_in)
            h = middlepad(**inputs)
            msg = ('Wrong shape in the output of middlepad with inputs ' +
                   str(inputs))
            self.assertEqual(shape_out, h.shape, msg)

    def test_default_val(self):
        """Check that expected default value is used
        """
        # check that default value for centering is 'wp'
        inputs_def = {}
        N = 3
        inputs_def['f'] = np.ones((N,))
        inputs_def['L'] = N+1
        h_def = middlepad(**inputs_def)
        inputs = deepcopy(inputs_def)
        inputs['centering'] = 'wp'
        h = middlepad(**inputs)
        msg = ('Wrong default value for centering when coparing inputs ' +
               str(inputs_def) + ' and ' + str(inputs))
        assert_array_equal(h_def, h, msg)

    def test_param_dim(self):
        """Check that dim is taken into account
        """
        inputs = {}
        N = 3
        M = 4
        inputs['L'] = 5
        inputs['f'] = np.ones((N, M))
        inputs['dim'] = 0
        h = middlepad(**inputs)
        msg = 'middlepad is misusing dim with inputs ' + str(inputs)
        self.assertEqual(h.shape, (inputs['L'], M), msg)
        inputs['dim'] = 1
        h = middlepad(**inputs)
        msg = 'middlepad is misusing dim with inputs ' + str(inputs)
        self.assertEqual(h.shape, (N, inputs['L']), msg)


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestMiddlepad)
    unittest.TextTestRunner(verbosity=2).run(suite)
